@@ -15,7 +15,7 @@ use crate::{
     errors::{KclError, KclErrorDetails},
     execution::{
         ExecState, GeometryWithImportedGeometry, KclValue, ModelingCmdMeta, Sketch, Solid,
-        types::{NumericType, PrimitiveType, RuntimeType},
+        types::{PrimitiveType, RuntimeType},
     },
     parsing::ast::types::TagNode,
     std::{Args, extrude::NamedCapTags},
@@ -64,7 +64,7 @@ async fn inner_clone(
         GeometryWithImportedGeometry::Solid(solid) => {
             // We flush before the clone so all the shit exists.
             exec_state
-                .flush_batch_for_solids((&args).into(), &[solid.clone()])
+                .flush_batch_for_solids((&args).into(), std::slice::from_ref(solid))
                 .await?;
 
             let mut new_solid = solid.clone();
@@ -145,10 +145,6 @@ async fn fix_tags_and_references(
             let new_solid = do_post_extrude(
                 &solid.sketch,
                 new_geometry_id.into(),
-                crate::std::args::TyF64::new(
-                    solid.height,
-                    NumericType::Known(crate::execution::types::UnitType::Length(solid.units)),
-                ),
                 solid.sectional,
                 &NamedCapTags {
                     start: start_tag.as_ref(),
