@@ -6,9 +6,7 @@ use kcmc::{
     length_unit::LengthUnit,
     shared::{Angle, Point2d as KPoint2d},
 };
-use kittycad_modeling_cmds as kcmc;
-use kittycad_modeling_cmds::shared::PathSegment;
-use schemars::JsonSchema;
+use kittycad_modeling_cmds::{self as kcmc, shared::PathSegment, units::UnitLength};
 use serde::Serialize;
 
 use super::{
@@ -20,7 +18,7 @@ use crate::{
     errors::{KclError, KclErrorDetails},
     execution::{
         BasePath, ExecState, GeoMeta, KclValue, ModelingCmdMeta, Path, Sketch, SketchSurface,
-        types::{RuntimeType, UnitLen},
+        types::{RuntimeType, adjust_length},
     },
     parsing::ast::types::TagNode,
     std::{
@@ -30,7 +28,7 @@ use crate::{
 };
 
 /// A sketch surface or a sketch.
-#[derive(Debug, Clone, Serialize, PartialEq, ts_rs::TS, JsonSchema)]
+#[derive(Debug, Clone, Serialize, PartialEq, ts_rs::TS)]
 #[ts(export)]
 #[serde(untagged)]
 pub enum SketchOrSurface {
@@ -87,7 +85,7 @@ async fn inner_rectangle(
             )));
         }
     };
-    let units = ty.expect_length();
+    let units = ty.as_length().unwrap_or(UnitLength::Millimeters);
     let corner_t = [TyF64::new(corner[0], ty), TyF64::new(corner[1], ty)];
 
     // Start the sketch then draw the 4 lines.
@@ -125,7 +123,8 @@ async fn inner_rectangle(
         .await?;
 
     // Update the sketch in KCL memory.
-    let mut new_sketch = sketch.clone();
+    let mut new_sketch = sketch;
+    new_sketch.is_closed = true;
     fn add(a: [f64; 2], b: [f64; 2]) -> [f64; 2] {
         [a[0] + b[0], a[1] + b[1]]
     }
@@ -180,7 +179,7 @@ async fn inner_circle(
         SketchOrSurface::Sketch(s) => s.on,
     };
     let (center_u, ty) = untype_point(center.clone());
-    let units = ty.expect_length();
+    let units = ty.as_length().unwrap_or(UnitLength::Millimeters);
 
     let radius = get_radius(radius, diameter, args.source_range)?;
     let from = [center_u[0] + radius.to_length_units(units), center_u[1]];
@@ -226,7 +225,8 @@ async fn inner_circle(
         ccw: angle_start < angle_end,
     };
 
-    let mut new_sketch = sketch.clone();
+    let mut new_sketch = sketch;
+    new_sketch.is_closed = true;
     if let Some(tag) = &tag {
         new_sketch.add_tag(tag, &current_path, exec_state);
     }
@@ -270,7 +270,7 @@ async fn inner_circle_three_point(
     args: Args,
 ) -> Result<Sketch, KclError> {
     let ty = p1[0].ty;
-    let units = ty.expect_length();
+    let units = ty.as_length().unwrap_or(UnitLength::Millimeters);
 
     let p1 = point_to_len_unit(p1, units);
     let p2 = point_to_len_unit(p2, units);
@@ -303,7 +303,7 @@ async fn inner_circle_three_point(
                     start: angle_start,
                     end: angle_end,
                     center: KPoint2d::from(untyped_point_to_mm(center, units)).map(LengthUnit),
-                    radius: units.adjust_to(radius, UnitLen::Mm).0.into(),
+                    radius: adjust_length(units, radius, UnitLength::Millimeters).0.into(),
                     relative: false,
                 },
             }),
@@ -327,7 +327,8 @@ async fn inner_circle_three_point(
         p3,
     };
 
-    let mut new_sketch = sketch.clone();
+    let mut new_sketch = sketch;
+    new_sketch.is_closed = true;
     if let Some(tag) = &tag {
         new_sketch.add_tag(tag, &current_path, exec_state);
     }
@@ -345,7 +346,7 @@ async fn inner_circle_three_point(
 }
 
 /// Type of the polygon
-#[derive(Debug, Clone, Serialize, PartialEq, ts_rs::TS, JsonSchema, Default)]
+#[derive(Debug, Clone, Serialize, PartialEq, ts_rs::TS, Default)]
 #[ts(export)]
 #[serde(rename_all = "lowercase")]
 pub enum PolygonType {
@@ -403,7 +404,7 @@ async fn inner_polygon(
     }
 
     let (sketch_surface, units) = match sketch_surface_or_group {
-        SketchOrSurface::SketchSurface(surface) => (surface, radius.ty.expect_length()),
+        SketchOrSurface::SketchSurface(surface) => (surface, radius.ty.as_length().unwrap_or(UnitLength::Millimeters)),
         SketchOrSurface::Sketch(group) => (group.on, group.units),
     };
 
@@ -522,17 +523,11 @@ async fn inner_polygon(
 
 /// Sketch an ellipse.
 pub async fn ellipse(exec_state: &mut ExecState, args: Args) -> Result<KclValue, KclError> {
-    exec_state.warn(crate::CompilationError {
-        source_range: args.source_range,
-        message: "Use of ellipse is currently experimental and the interface may change.".to_string(),
-        suggestion: None,
-        severity: crate::errors::Severity::Warning,
-        tag: crate::errors::Tag::None,
-    });
     let sketch_or_surface =
         args.get_unlabeled_kw_arg("sketchOrSurface", &RuntimeType::sketch_or_surface(), exec_state)?;
     let center = args.get_kw_arg("center", &RuntimeType::point2d(), exec_state)?;
-    let major_radius = args.get_kw_arg("majorRadius", &RuntimeType::length(), exec_state)?;
+    let major_radius = args.get_kw_arg_opt("majorRadius", &RuntimeType::length(), exec_state)?;
+    let major_axis = args.get_kw_arg_opt("majorAxis", &RuntimeType::point2d(), exec_state)?;
     let minor_radius = args.get_kw_arg("minorRadius", &RuntimeType::length(), exec_state)?;
     let tag = args.get_kw_arg_opt("tag", &RuntimeType::tag_decl(), exec_state)?;
 
@@ -540,6 +535,7 @@ pub async fn ellipse(exec_state: &mut ExecState, args: Args) -> Result<KclValue,
         sketch_or_surface,
         center,
         major_radius,
+        major_axis,
         minor_radius,
         tag,
         exec_state,
@@ -551,10 +547,12 @@ pub async fn ellipse(exec_state: &mut ExecState, args: Args) -> Result<KclValue,
     })
 }
 
+#[allow(clippy::too_many_arguments)]
 async fn inner_ellipse(
     sketch_surface_or_group: SketchOrSurface,
     center: [TyF64; 2],
-    major_radius: TyF64,
+    major_radius: Option<TyF64>,
+    major_axis: Option<[TyF64; 2]>,
     minor_radius: TyF64,
     tag: Option<TagNode>,
     exec_state: &mut ExecState,
@@ -565,9 +563,29 @@ async fn inner_ellipse(
         SketchOrSurface::Sketch(group) => group.on,
     };
     let (center_u, ty) = untype_point(center.clone());
-    let units = ty.expect_length();
+    let units = ty.as_length().unwrap_or(UnitLength::Millimeters);
 
-    let from = [center_u[0] + major_radius.to_length_units(units), center_u[1]];
+    let major_axis = match (major_axis, major_radius) {
+        (Some(_), Some(_)) | (None, None) => {
+            return Err(KclError::new_type(KclErrorDetails::new(
+                "Provide either `majorAxis` or `majorRadius`.".to_string(),
+                vec![args.source_range],
+            )));
+        }
+        (Some(major_axis), None) => major_axis,
+        (None, Some(major_radius)) => [
+            major_radius.clone(),
+            TyF64 {
+                n: 0.0,
+                ty: major_radius.ty,
+            },
+        ],
+    };
+
+    let from = [
+        center_u[0] + major_axis[0].to_length_units(units),
+        center_u[1] + major_axis[1].to_length_units(units),
+    ];
     let from_t = [TyF64::new(from[0], ty), TyF64::new(from[1], ty)];
 
     let sketch =
@@ -578,6 +596,7 @@ async fn inner_ellipse(
 
     let id = exec_state.next_uuid();
 
+    let axis = KPoint2d::from(untyped_point_to_mm([major_axis[0].n, major_axis[1].n], units)).map(LengthUnit);
     exec_state
         .batch_modeling_cmd(
             ModelingCmdMeta::from_args_id(&args, id),
@@ -585,7 +604,7 @@ async fn inner_ellipse(
                 path: sketch.id.into(),
                 segment: PathSegment::Ellipse {
                     center: KPoint2d::from(point_to_mm(center)).map(LengthUnit),
-                    major_radius: LengthUnit(major_radius.to_mm()),
+                    major_axis: axis,
                     minor_radius: LengthUnit(minor_radius.to_mm()),
                     start_angle: Angle::from_degrees(angle_start.to_degrees()),
                     end_angle: Angle::from_degrees(angle_end.to_degrees()),
@@ -605,13 +624,14 @@ async fn inner_ellipse(
                 metadata: args.source_range.into(),
             },
         },
-        major_radius: major_radius.to_length_units(units),
+        major_axis: major_axis.map(|x| x.to_length_units(units)),
         minor_radius: minor_radius.to_length_units(units),
         center: center_u,
         ccw: angle_start < angle_end,
     };
 
-    let mut new_sketch = sketch.clone();
+    let mut new_sketch = sketch;
+    new_sketch.is_closed = true;
     if let Some(tag) = &tag {
         new_sketch.add_tag(tag, &current_path, exec_state);
     }
